@@ -12,6 +12,7 @@
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE GADTs                 #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE NamedFieldPuns        #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
@@ -684,6 +685,18 @@ instance (SymVal a, SatisfiableM m p) => SatisfiableM m (SBV a -> p) where
 instance (SymVal a, ProvableM m p) => ProvableM m (SBV a -> p) where
   proofArgReduce fn = mkArg >>= \a -> proofArgReduce $ fn a
 
+-- | Create an 'SBVs' sequence of arguments
+mkArgs :: MonadSymbolic m => SymValInsts as -> m (SBVs as)
+mkArgs SymValsNil = return SBVsNil
+mkArgs (SymValsCons insts) = SBVsCons <$> mkArgs insts <*> mkArg
+
+-- Multi-arity Functions
+instance (SymVals as, SatisfiableM m p) => SatisfiableM m (SBVs as -> p) where
+  satArgReduce fn = mkArgs symValInsts >>= \args -> satArgReduce $ fn args
+
+instance (SymVals as, ProvableM m p) => ProvableM m (SBVs as -> p) where
+  proofArgReduce fn = mkArgs symValInsts >>= \args -> proofArgReduce $ fn args
+
 -- 2 Tuple
 instance (SymVal a, SymVal b, SatisfiableM m p) => SatisfiableM m ((SBV a, SBV b) -> p) where
   satArgReduce fn = mkArg >>= \a -> satArgReduce $ \b -> fn (a, b)
