@@ -9,11 +9,9 @@
 -- Abstraction of SMT solvers
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DefaultSignatures          #-}
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE NamedFieldPuns             #-}
-{-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE Rank2Types                 #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
 {-# LANGUAGE TypeApplications           #-}
@@ -27,7 +25,7 @@ module Data.SBV.SMT.SMT (
          Modelable(..)
        , SatModel(..), genParse
        , extractModels, getModelValues
-       , getModelDictionaries, getModelUninterpretedValues
+       , getModelDictionaries
        , displayModels, showModel, shCV, showModelDictionary
 
        -- * Standard prover engine
@@ -234,14 +232,13 @@ class SatModel a where
   -- the remaining elements untouched. If the next element is not what's expected for this
   -- type you should return 'Nothing'
   parseCVs  :: [CV] -> Maybe (a, [CV])
+
   -- | Given a parsed model instance, transform it using @f@, and return the result.
   -- The default definition for this method should be sufficient in most use cases.
   cvtModel  :: (a -> Maybe b) -> Maybe (a, [CV]) -> Maybe (b, [CV])
   cvtModel f x = x >>= \(a, r) -> f a >>= \b -> return (b, r)
 
-  default parseCVs :: Read a => [CV] -> Maybe (a, [CV])
-  parseCVs (CV _ (CUserSort (_, s)) : r) = Just (read s, r)
-  parseCVs _                             = Nothing
+  {-# MINIMAL parseCVs #-}
 
 -- | Parse a signed/sized value from a sequence of CVs
 genParse :: Integral a => Kind -> [CV] -> Maybe (a, [CV])
@@ -328,7 +325,12 @@ instance SatModel CV where
   parseCVs []       = Nothing
 
 -- | A rounding mode, extracted from a model. (Default definition suffices)
-instance SatModel RoundingMode
+instance SatModel RoundingMode where
+  parseCVs (CV k (CADT (s, [])) : r)
+    | isRoundingMode k
+    , Just mode <- s `lookup` [(show m, m) | m <- [minBound .. maxBound :: RoundingMode]]
+    = Just (mode, r)
+  parseCVs _ = Nothing
 
 -- | 'String' as extracted from a model
 instance {-# OVERLAPS #-} SatModel [Char] where
@@ -404,14 +406,6 @@ class Modelable a where
   getModelValue :: SymVal b => String -> a -> Maybe b
   getModelValue v r = fromCV `fmap` (v `M.lookup` getModelDictionary r)
 
-  -- | Extract a representative name for the model value of an uninterpreted kind.
-  -- This is supposed to correspond to the value as computed internally by the
-  -- SMT solver; and is unportable from solver to solver. Also see `getModelUninterpretedValues`.
-  getModelUninterpretedValue :: String -> a -> Maybe String
-  getModelUninterpretedValue v r = case v `M.lookup` getModelDictionary r of
-                                     Just (CV _ (CUserSort (_, s))) -> Just s
-                                     _                              -> Nothing
-
   -- | A simpler variant of 'getModelAssignment' to get a model out without the fuss.
   extractModel :: SatModel b => a -> Maybe b
   extractModel a = case getModelAssignment a of
@@ -445,10 +439,6 @@ getModelDictionaries AllSatResult{allSatResults = xs} = map getModelDictionary x
 getModelValues :: SymVal b => String -> AllSatResult -> [Maybe b]
 getModelValues s AllSatResult{allSatResults = xs} =  map (s `getModelValue`) xs
 
--- | Extract value of an uninterpreted variable from an all-sat call. Similar to `getModelUninterpretedValue`.
-getModelUninterpretedValues :: String -> AllSatResult -> [Maybe String]
-getModelUninterpretedValues s AllSatResult{allSatResults = xs} =  map (s `getModelUninterpretedValue`) xs
-
 -- | t'ThmResult' as a generic model provider
 instance Modelable ThmResult where
   getModelAssignment (ThmResult r) = getModelAssignment r
