@@ -10,13 +10,10 @@
 -- by properly defining your metric values.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
-{-# LANGUAGE TemplateHaskell     #-}
-{-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE TypeApplications  #-}
+{-# LANGUAGE TypeFamilies      #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -26,10 +23,9 @@ import Data.SBV
 
 -- | A simple enumeration
 data Day = Mon | Tue | Wed | Thu | Fri | Sat | Sun
-         deriving (Enum, Bounded)
 
 -- | Make 'Day' a symbolic value.
-mkSymbolicEnumeration ''Day
+mkSymbolic [''Day]
 
 -- | Make day an optimizable value, by mapping it to 'Word8' in the most
 -- obvious way. We can map it to any value the underlying solver can optimize,
