@@ -13,9 +13,8 @@
 {-# LANGUAGE DataKinds           #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE OverloadedLists     #-}
-{-# LANGUAGE TypeAbstractions    #-}
-{-# LANGUAGE TypeApplications    #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -142,7 +141,7 @@ correctness = runTPWith (tpRibbon 60 z3) $ do
     mergeKeepsSort <-
         sInductWith cvc5 "mergeKeepsSort"
            (\(Forall xs) (Forall ys) -> nonDecreasing xs .&& nonDecreasing ys .=> nonDecreasing (merge xs ys))
-           (\xs ys -> tuple (length xs, length ys)) $
+           (\xs ys -> tuple (length xs, length ys), []) $
            \ih xs ys -> [nonDecreasing xs, nonDecreasing ys]
                      |- split2 (xs, ys)
                                trivial           -- when both xs and ys are empty.  Trivial.
@@ -170,7 +169,7 @@ correctness = runTPWith (tpRibbon 60 z3) $ do
     sortNonDecreasing <-
         sInduct "sortNonDecreasing"
                 (\(Forall xs) -> nonDecreasing (mergeSort xs))
-                length $
+                (length, []) $
                 \ih xs -> [] |- split xs
                                       qed
                                       (\e es -> nonDecreasing (mergeSort (e .: es))
@@ -199,7 +198,7 @@ correctness = runTPWith (tpRibbon 60 z3) $ do
     mergeCount <-
         sInduct "mergeCount"
                 (\(Forall xs) (Forall ys) (Forall e) -> count e (merge xs ys) .== count e xs + count e ys)
-                (\xs ys _e -> tuple (length xs, length ys)) $
+                (\xs ys _e -> tuple (length xs, length ys), []) $
                 \ih as bs e -> [] |- split2 (as, bs)
                                             trivial
                                             trivial
@@ -236,7 +235,7 @@ correctness = runTPWith (tpRibbon 60 z3) $ do
     sortIsPermutation <-
         sInductWith cvc5 "sortIsPermutation"
                 (\(Forall xs) (Forall e) -> count e xs .== count e (mergeSort xs))
-                (\xs _e -> length xs) $
+                (\xs _e -> length xs, []) $
                 \ih as e -> [] |- split as
                                         trivial
                                         (\x xs -> count e (mergeSort (x .: xs))
